#!/usr/bin/env python

'''
'''

__docformat__ = 'restructuredtext'
__version__ = '$Id: $'

import math
import random

from gamelib.GameObject import GameObject
from gamelib.Skeleton import Skeleton

from gamelib import constants as C
from gamelib import glob

C0 = (243, 198, 182)
C1 = (233, 150, 122)

class Audience(GameObject):
    def __init__(self, level):
        self.skeleton = sk = Skeleton('audience')
        sk.root('Torso')
        sk.connect('Torso', 'LeftLeg')
        sk.connect('LeftLeg', 'LeftFoot')
        sk.connect('Torso', 'RightLeg')
        sk.connect('RightLeg', 'RightFoot')

        self.base_y = 200 + level

        level_f = (100 - level) / 100.
        color = tuple(int(C1[i] * level_f + C0[i] * (1 - level_f)) 
                      for i in range(3))
        for bone in sk.draw_order:
            bone.color = color

        self.variance = (random.random() + 1) * C.AUDIENCE_VARIANCE

    def reset(self):
        self.state = 'oob'
        self.leg_swing = 0
        self.skeleton.set_position(-C.AUDIENCE_BOUNDS - random.randrange(0, 800), 0)

    def draw(self):
        self.skeleton.draw()

    def update(self, dt):
        dt *= self.variance
        sk = self.skeleton

        if self.state == 'stopped':
            sk['LeftFoot'].flip_x = True
            sk['LeftLeg'].rotation = C.AUDIENCE_STAND_LEG_ROTATION
            sk['RightLeg'].rotation = -C.AUDIENCE_STAND_LEG_ROTATION
            sk['LeftFoot'].rotation = -C.AUDIENCE_STAND_LEG_ROTATION
            sk['RightFoot'].rotation = C.AUDIENCE_STAND_LEG_ROTATION

            self.countdown_stay -= 1
            if self.countdown_stay < 0:
                if random.random() < 1 - C.AUDIENCE_STAY_CHANCE * glob.last_entertainment:
                    self.state = 'walking'
                    glob.audience_standing_count -= 1
                self.countdown_stay = C.AUDIENCE_STAY_COUNTDOWN

        elif self.state == 'stopping' :
            if abs(math.sin(self.leg_swing)) < C.AUDIENCE_LEG_SWING_STOPPED:
                self.state = 'stopped'
                self.countdown_stay = C.AUDIENCE_STAY_COUNTDOWN
                glob.audience_standing_count += 1

        if self.state == 'walking' or self.state == 'stopping':
            sk['LeftFoot'].flip_x = False

            if random.random() < C.AUDIENCE_STOP_CHANCE * glob.last_entertainment:
                self.state = 'stopping'

            self.leg_swing += dt * C.AUDIENCE_LEG_SWING_SPEED
            x = self.x + dt * C.AUDIENCE_WALK_SPEED
            if x > glob.window.width + C.AUDIENCE_BOUNDS:
                x = -C.AUDIENCE_BOUNDS - 1
                self.state = 'oob'
                glob.audience_count -= 1

            sk.set_position(x, self.base_y - C.AUDIENCE_TORSO_BOUNCE * math.sin(2 * self.leg_swing))
            sk['Torso'].rotation = C.AUDIENCE_TORSO_SWING * math.sin(2 * self.leg_swing)
            sk['LeftLeg'].rotation = C.AUDIENCE_LEG_SWING * math.sin(self.leg_swing)
            sk['RightLeg'].rotation = -C.AUDIENCE_LEG_SWING * math.sin(self.leg_swing)
            sk['LeftFoot'].rotation = min(0, -C.AUDIENCE_LEG_SWING * math.sin(self.leg_swing))
            sk['RightFoot'].rotation = min(0, C.AUDIENCE_LEG_SWING * math.sin(self.leg_swing))

        if self.state == 'oob':
            if random.random() < min(max(C.AUDIENCE_ENTER_CHANCE_MIN,
                                         C.AUDIENCE_ENTER_CHANCE * glob.last_entertainment),
                                     C.AUDIENCE_ENTER_CHANCE_MAX):
                self.state = 'walking'
                glob.audience_count += 1

        self.skeleton.update(dt)
