#!/usr/bin/env python

'''
'''

__docformat__ = 'restructuredtext'
__version__ = '$Id: $'

import pyglet
from pyglet.gl import *

from gamelib.SizeableSprite import SizeableSprite

class Button(object):
    def __init__(self, x1, y1, x2, y2, batch=None):
        if batch is None:
            batch = self.batch = pyglet.graphics.Batch()
        
        corner = pyglet.resource.image('button_Corner.png')
        fill = pyglet.resource.image('button_Fill.png')

        glBindTexture(fill.texture.target, fill.texture.id)
        glTexParameteri(fill.texture.target, GL_TEXTURE_WRAP_S, GL_CLAMP)
        glTexParameteri(fill.texture.target, GL_TEXTURE_WRAP_T, GL_CLAMP)

        if not hasattr(fill, 'has_fixed_tex_coords'):
            u = 1 / float(fill.texture.width)
            v = 1 / float(fill.texture.height)
            u1 = fill.tex_coords[0] + u
            v1 = fill.tex_coords[1] + v
            u2 = fill.tex_coords[3] - u
            v2 = fill.tex_coords[7] - v
            r = 0
            fill.tex_coords = (u1, v1, r, u2, v1, r, u2, v2, r, u1, v2, r)
            fill.has_fixed_tex_coords = True

        self.corner_w = corner.width
        self.corner_h = corner.height

        def sprite(img):
            return SizeableSprite(img, 
                                  blend_src=GL_ONE,
                                  blend_dest=GL_ONE_MINUS_SRC_ALPHA, 
                                  batch=batch)

        self.corners = [
            sprite(corner),
            sprite(corner),
            sprite(corner),
            sprite(corner)
        ]

        self.fills = [
            sprite(fill),
            sprite(fill),
            sprite(fill),
            sprite(fill),
            sprite(fill),
        ]

        self.set_position(x1, y1, x2, y2)

        self.set_color((127, 127, 127, 127))

    def set_position(self, x1, y1, x2, y2):
        w = self.corner_w
        h = self.corner_h

        self.corners[0].set_position(x1, y1 + h, x1 + w, y1) # bl
        self.corners[1].set_position(x1, y2 - h, x1 + w, y2) # tl
        self.corners[2].set_position(x2, y1 + h, x2 - w, y1) # br
        self.corners[3].set_position(x2, y2 - h, x2 - w, y2) # tr

        self.fills[0].set_position(x1, y1 + h, x1 + w, y2 - h) # l
        self.fills[1].set_position(x2 - w, y1 + h, x2, y2 - h) # r
        self.fills[2].set_position(x1 + w, y1 + h, x2 - w, y2 - h) # c 
        self.fills[3].set_position(x1 + w, y1, x2 - w, y1 + h) # b 
        self.fills[4].set_position(x1 + w, y2 - h, x2 - w, y2) # t

    def set_color(self, color):
        opacity = color[3]
        color = tuple(c * opacity / 255 for c in color[:3])
        map(lambda s:s._set_color(color), self.corners)
        map(lambda s:s._set_color(color), self.fills)
        map(lambda s:s._set_opacity(opacity), self.corners)
        map(lambda s:s._set_opacity(opacity), self.fills)

    def draw(self):
        self.batch.draw()
