#!/usr/bin/env python

'''
'''

__docformat__ = 'restructuredtext'
__version__ = '$Id: $'

import random

import pyglet

from gamelib.GameObject import GameObject
from gamelib.BoneSprite import BoneSprite

from gamelib import glob
from gamelib import constants as C

class Coin(GameObject):
    can_throw = False # wait until tutorial
    did_throw = None
    did_pickup = None

    def __init__(self):
        self.img_silver = pyglet.resource.image('silver_coin.png')
        self.img_gold = pyglet.resource.image('gold_coin.png')

        self.sprite = BoneSprite(self.img_silver, batch=glob.window.batch)

    def reset(self):
        self.sprite.y = -100
        self.state = 'unconstructed'
        self.sprite.color = 255,255, 255
        self.sprite.opacity = 255
        Coin.can_throw = False

    def throw(self):
        if not self.can_throw:
            return
        self.sprite.x = random.randrange(C.COIN_X_MIN, C.COIN_X_MAX)
        self.last_y = self.sprite.y = C.COIN_DROP_Y
        self.state = 'falling'
        if (random.random() < C.COIN_THROW_CHANCE_GOLD *
            glob.last_entertainment):
            self.sprite.image = self.img_gold
            self.value = 1.00
        else:
            self.sprite.image = self.img_silver
            self.value = 0.20
        if self.did_throw:
            self.did_throw()

    def pickup(self):
        self.state = 'rising'
        glob.cash += self.value
        glob.window.play('wine-glass-clink-2.wav', over=True)

    def update(self, dt):
        if (self.state == 'unconstructed' and 
            random.random() < C.COIN_THROW_CHANCE * glob.audience_count +
                C.COIN_THROW_CHANCE_STANDING * glob.audience_standing_count):
            self.throw()
        elif self.state == 'falling':
            last_y = self.sprite.y
            self.sprite.y = 2 * last_y - self.last_y + C.GRAVITY * (C.GRAVITY_TIMESTEP_2)
            self.last_y = last_y
            if self.sprite.y < C.COIN_Y:
                self.sprite.y = C.COIN_Y
                self.state = 'idle'
                glob.window.play(random.choice((
                    'coin_1.wav',
                    'coin_2.wav',
                    'coin_4.wav',
                    'coin_5.wav',
                )), over=True)
        elif self.state == 'rising':
            self.sprite.y += dt * C.COIN_PICKUP_DY
            self.sprite.opacity -= dt * 500
            o = self.sprite.opacity
            self.sprite.color = (o, o, o)
            if self.sprite.y > C.COIN_PICKUP_Y:
                self.state = 'unconstructed'
                self.sprite.y = -100
                self.sprite.opacity = 255
                self.sprite.color = (255, 255, 255)
                if self.did_pickup:
                    self.did_pickup()
        elif (self.state == 'idle' and
            abs(glob.player.x - self.sprite.x) < C.COIN_PICKUP_X):
            self.pickup()
