#!/usr/bin/env python

'''
'''

__docformat__ = 'restructuredtext'
__version__ = '$Id: $'

import pyglet

from pyglet.window import key

from gamelib.Button import Button
from gamelib import constants as C
from gamelib import glob

class Dialog(object):
    def __init__(self, text):
        self.batch = pyglet.graphics.Batch()
        m = C.DIALOG_MARGIN
        self.bg = Button(m, m, glob.window.width - m, glob.window.height - m, self.batch) 
        self.bg.set_color(C.DIALOG_COLOR_BG)

        self.title = pyglet.text.Label(text,
                                       font_name=C.FONT_NAME,
                                       font_size=18., 
                                       x=glob.window.width/2,
                                       y=glob.window.height - m - C.DIALOG_PAD_Y,
                                       anchor_x='center',
                                       anchor_y='top',
                                       color=(0, 0, 0, 255),
                                       batch=self.batch)

        self.items = []
        self.selected_index = -1

    def add(self, text, func):
        self.items.append(Item(len(self.items) + 1, text, self.batch, func))

    def show(self):
        self.select(0)
        glob.window.push_dialog(self)

    def hide(self):
        glob.window.pop_dialog()

    def select(self, index):
        self.selected_index = index
        for i, item in enumerate(self.items):
            if i == index and item.state != 'disabled':
                item.set_state('selected')
            elif item.state == 'selected':
                item.set_state('idle')

    def move_select(self, dir):
        start = self.selected_index % len(self.items)
        while True:
            self.selected_index = (self.selected_index + dir) % len(self.items)
            if self.selected_index == start:
                break
            elif self.items[self.selected_index].state != 'disabled':
                self.select(self.selected_index)
                break

    def draw(self):
        self.batch.draw()

    def on_key_press(self, symbol, modifiers):
        if symbol == key.DOWN:
            self.move_select(1)
        elif symbol == key.UP:
            self.move_select(-1)
        elif symbol == key.ENTER:
            if self.selected_index != -1:
                self.hide()
                self.items[self.selected_index].func()
        return True

    def on_mouse_motion(self, x, y, dx, dy):
        for i, item in enumerate(self.items):
            if item.hit_test(x, y):
                self.select(i)
                break
        else:
            self.select(-1)

    def on_mouse_press(self, x, y, button, modifiers):
        for i, item in enumerate(self.items):
            if item.hit_test(x, y) and item.state != 'disabled':
                self.hide()
                item.func()
                break
        
class Item(object):
    def __init__(self, i, name, batch, func):
        self.func = func

        y2 = glob.window.height - C.DIALOG_MARGIN - C.DIALOG_PAD_Y - \
            i * (C.DIALOG_ITEM_LEADING + C.DIALOG_ITEM_HEIGHT)
        y1 = y2 - C.DIALOG_ITEM_HEIGHT
        x1 = C.DIALOG_MARGIN + C.DIALOG_PAD_X
        x2 = glob.window.width - C.DIALOG_MARGIN - C.DIALOG_PAD_X
        self.x1 = x1
        self.y1 = y1
        self.x2 = x2
        self.y2 = y2
        self.button = Button(x1, y1, x2, y2, batch)

        self.label = pyglet.text.Label(name, 
                                       font_name=C.FONT_NAME,
                                       font_size=20., 
                                       anchor_x='center',
                                       anchor_y='center',
                                       color=(0, 0, 0, 255),
                                       batch=batch)
        self.label.x = (x1 + x2) / 2
        self.label.y = (y1 + y2) / 2 + 4 # fudge vcenter
        self.set_state('idle')

    def set_state(self, state):
        self.state = state
        if state == 'idle':
            self.button.set_color(C.DIALOG_ITEM_COLOR_BG)
        elif state == 'selected':
            self.button.set_color(C.DIALOG_ITEM_COLOR_BG_SELECTED)
        elif state == 'disabled':
            self.button.set_color(C.DIALOG_ITEM_COLOR_BG_DISABLED)
        
    def hit_test(self, x, y):
        return (self.x1 < x < self.x2) and (self.y1 < y < self.y2)
