#!/usr/bin/env python

'''
'''

__docformat__ = 'restructuredtext'
__version__ = '$Id: $'

import pyglet

from gamelib.Coin import Coin
from gamelib.Plate import Plate
from gamelib.SpinningPlate import SpinningPlate

from gamelib import constants as C
from gamelib import glob

class Hints(object):
    def __init__(self):
        self.label = pyglet.text.Label('',
            font_name=C.FONT_NAME,
            font_size=16.,
            x=glob.window.width / 2,
            y=C.HINT_Y,
            anchor_x='center',
            anchor_y='baseline',
            color=(0, 0, 0, 0))

        def get_opacity(self):
            return self.color[3]
        def set_opacity(self, alpha):
            r, g, b, _ = self.color
            self.color = (r, g, b, int(alpha))
        pyglet.text.Label.opacity = property(get_opacity, set_opacity)

        self.queue = []
        self.state = None
        self.wait_time = 0
        pyglet.clock.schedule_interval(self.update, 1/30.)

    def reset(self):
        self.queue = []
        self.label.opacity = 0

        glob.player.did_move = None
        glob.player.did_stop = None
        glob.player.did_turn = None
        glob.window.did_store = None
        Coin.did_throw = None
        Coin.did_pickup = None
        Plate.did_pickup = None
        SpinningPlate.did_purchase = None
        SpinningPlate.did_fall = None
        SpinningPlate.did_start = None

    def start(self):
        self.wait(2., self.hint_move)

    def draw(self):
        self.label.draw()

    def update(self, dt):
        if self.state == 'fadein':
            self.label.opacity += dt * 500
            if self.label.opacity >= 255:
                self.label.opacity = 255
                self._next()
        elif self.state == 'fadeout':
            self.label.opacity -= dt * 500
            if self.label.opacity <= 0:
                self.label.opacity = 0
                self._next()
        elif self.state == 'wait':
            self.wait_time -= dt
            if self.wait_time < 0:
                self._next()
        else:
            self._next()

    def _next(self):
        self.state = None
        if self.queue:
            self.queue.pop(0)()

    def fade_out(self):
        def doit():
            self.state = 'fadeout'
        self.queue.append(doit)

    def fade_in(self):
        def doit():
            self.state = 'fadein'
        self.queue.append(doit)

    def wait(self, time, next_func=None):
        def doit():
            self.wait_time = time
            self.state = 'wait'
        self.queue.append(doit)
        if next_func:
            self.queue.append(next_func)

    def set(self, text):
        def doit():
            self.label.text = text

        self.fade_out()
        self.queue.append(doit)
        self.wait(1)
        self.fade_in()

    def hint_move(self):
        self.set('Move left and right with the ARROW keys.')
        glob.player.did_move = self.did_move

    def did_move(self):
        glob.player.did_move = None
        self.wait(5., self.hint_stop)

    def hint_stop(self):
        self.set('Press and hold DOWN to stop quickly while cycling')
        glob.player.did_stop = self.did_stop

    def did_stop(self):
        glob.player.did_stop = None
        self.wait(.5, self.hint_turn)

    def hint_turn(self):
        self.set('Press the opposite direction while stopping to turn around')
        glob.player.did_turn = self.did_turn

    def did_turn(self):
        glob.player.did_turn = None
        self.fade_out()

        # begin game
        Coin.can_throw = True
        Coin.did_throw = self.did_coin_throw

    def did_coin_throw(self):
        Coin.did_throw = None
        self.hint_coin_throw()

    def hint_coin_throw(self):
        self.set('Roll over coins to pick them up')
        Coin.did_pickup = self.did_coin_pickup

    def did_coin_pickup(self):
        Coin.did_pickup = self.did_coin_pickup_wait_store
        self.fade_out()

    def did_coin_pickup_wait_store(self):
        if glob.cash >= C.PRICE_SPINNING_POLE:
            Coin.did_pickup = None
            self.hint_store()

    def hint_store(self):
        self.set('Press SPACEBAR to buy new equipment')
        glob.window.did_store = self.did_store

    def did_store(self):
        self.label.text = 'Purchase a spinning pole to improve your act'
        self.label.opacity = 255
        self.queue = []
        glob.window.did_store = None
        SpinningPlate.did_purchase = self.did_purchase_spinning_pole

    def did_purchase_spinning_pole(self):
        SpinningPlate.did_purchase = None
        self.set('Pick up a plate from the right')
        Plate.did_pickup = self.did_pickup_plate

    def did_pickup_plate(self):
        Plate.did_pickup = None
        self.set('Stand still next to the pole to spin the plate up to speed')
        SpinningPlate.did_fall = self.did_spinning_plate_fall
        SpinningPlate.did_start = self.did_spinning_plate_start

    def did_spinning_plate_fall(self):
        SpinningPlate.did_fall = None
        self.queue = []
        self.set('Oops!  Keep spinning the pole to stop the plate falling off')
        self.wait(2, self.fade_out)

    def did_spinning_plate_start(self):
        glob.window.play('small_studio_crowd_applause.mp3')
        SpinningPlate.did_start = None
        self.queue = []
        self.set('Well done!  Remember to keep this plate spinning')
        self.wait(2, self.hint_equipment)

    def hint_equipment(self):
        self.set('Purchase more equipment to improve your act and earn more money')
        self.wait(2, self.fade_out)


    


