#!/usr/bin/env python

'''
'''

__docformat__ = 'restructuredtext'
__version__ = '$Id: $'

import math

import pyglet
from pyglet.window import key

from gamelib.BoneSprite import BoneSprite
from gamelib.GameObject import GameObject
from gamelib.Skeleton import Skeleton

from gamelib import constants as C
from gamelib import glob

from res import man_vectors

class Player(GameObject):
    def __init__(self):
        self.plate = BoneSprite(pyglet.resource.image('plate_Plate.png'), 
                                batch=glob.window.batch)
        self.plate.y = -1000

        self.skeleton = sk = Skeleton('man')
        sk.add('LeftFoot')
        sk.add('LeftLeg')
        sk.root('Seat')
        sk.connect('Seat', 'LeftLeg')
        sk.connect('LeftLeg', 'LeftFoot')
        sk.root('Wheel')
        sk.add('RightFoot')
        sk.add('RightLeg')
        sk.connect('Seat', 'RightLeg')
        sk.connect('RightLeg', 'RightFoot')
        sk.connect('Seat', 'Bum')
        sk.add('LeftHand')
        sk.add('LeftArm')
        sk.connect('LeftArm', 'LeftHand')
        sk.add('Head')
        sk.connect('Torso', 'LeftArm')
        sk.connect('Bum', 'Torso')
        sk.add('RightHand')
        sk.add('RightArm')
        sk.connect('RightArm', 'RightHand')
        sk.connect('Torso', 'RightArm')
        sk.connect('Torso', 'Head')

        self.wheel = sk['Wheel']
        self.wheel.radius = self.wheel._bone_size
        self.wheel.circumference = 2 * math.pi * self.wheel.radius

        self.keys = key.KeyStateHandler()
        self.reset()

    def reset(self):
        self.wheel_dx = C.WHEEL_INIT
        self.wheel_offset = 0
        self.wheel_can_break = False
        self.wheel_is_breaking = False
        self.wheel_brake_dx = 1
        self.wheel_y_prev = self.skeleton.y
        self.wheel_can_turn = False
        self.has_plate = False
        self.plate.bone_parent = None
        self.plate.y = -1000
        self.active_object = None
        self.skeleton.flip_x = False

        self.did_move = None
        self.did_stop = None
        self.did_turn = None

    def update(self, dt):
        anykey = False
        sk = self.skeleton

        ground = C.GROUND + self.wheel.radius

        airborne = sk.y > ground
        
        # Breaking
        if self.keys[key.DOWN]:
            if ((self.wheel_can_break and abs(self.wheel_dx) > C.WHEEL_BRAKE_START_THRESHOLD) or 
                (self.wheel_is_breaking and abs(self.wheel_dx) > C.WHEEL_BRAKE_END_THRESHOLD)):

                if not self.wheel_is_breaking:
                    self.wheel_can_turn = True
                    glob.window.play('break.wav', over=True)
                    if self.did_stop:
                        self.did_stop()

                self.wheel_dx -= C.WHEEL_BRAKE_FRICTION * self.wheel_dx * dt
                self.wheel_offset = -self.wheel.x - self.wheel.circumference / 4
                self.wheel_can_break = False
                self.wheel_is_breaking = True
                anykey = True

                if (self.wheel_can_turn and 
                    ((self.keys[key.LEFT] and self.wheel_dx > 0) or
                     (self.keys[key.RIGHT] and self.wheel_dx < 0))):
                    self.wheel_can_turn = False
                    sk.flip_x = not sk.flip_x
                    self.wheel_y_prev = sk.y - C.WHEEL_TURN_JUMP
                    sk.y += 1
                    airborne = True
                    glob.entertainment += C.ENTERTAINMENT_TURN
                    if self.did_turn:
                        self.did_turn()

                if not airborne:
                    # vertical impulse
                    self.wheel_y_prev = sk.y - C.WHEEL_BRAKE_JUMP
                    sk.y += 1

            else:
                # Finished breaking
                self.wheel_is_breaking = False
                self.wheel_can_turn = False

        # Cycling
        elif not airborne:
            self.wheel_can_break = True
            self.wheel_is_breaking = False
            if self.keys[key.LEFT] and sk.x > C.WHEEL_BOUNDS_MARGIN:
                self.wheel_dx -= dt * C.WHEEL_ACCEL
                anykey = True
                if self.did_move:
                    self.did_move()
            if self.keys[key.RIGHT] and sk.x < glob.window.width - C.WHEEL_BOUNDS_MARGIN:
                self.wheel_dx += dt * C.WHEEL_ACCEL
                anykey = True
                if self.did_move:
                    self.did_move()

        # Stability
        if not anykey and not airborne:
            stability = math.sin(2 * (self.wheel._rotation + math.pi / 4))
            if not self.active_object:
                self.wheel_dx -= stability * dt * C.WHEEL_STABILITY_ACCEL

            if abs(self.wheel_dx) > C.WHEEL_FRICTION_THRESHOLD:
                self.wheel_dx -= C.WHEEL_FRICTION * self.wheel_dx * dt

            if self.active_object and abs(self.wheel_dx) > C.WHEEL_FRICTION_THRESHOLD_ACTIVE:
                self.wheel_dx -= C.WHEEL_FRICTION_ACTIVE * self.wheel_dx * dt

        if sk.x < C.WHEEL_BOUNDS_HARD_MARGIN and self.wheel_dx < 0:
            self.wheel_dx *= -1
        elif sk.x > glob.window.width - C.WHEEL_BOUNDS_HARD_MARGIN and self.wheel_dx > 0:
            self.wheel_dx *= -1
        elif sk.x < C.WHEEL_BOUNDS_MARGIN and self.wheel_dx < 0:
            self.wheel_dx -= self.wheel_dx * dt * C.WHEEL_BOUNDS_ACCEL
        elif sk.x > glob.window.width - C.WHEEL_BOUNDS_MARGIN and self.wheel_dx > 0:
            self.wheel_dx -= self.wheel_dx * dt * C.WHEEL_BOUNDS_ACCEL

        if airborne:
            glob.entertainment += C.ENTERTAINMENT_AIRBORNE

        wheel = self.wheel
        x = sk.x + self.wheel_dx * dt * C.WHEEL_SPEED

        # Verlet y displacement
        y = sk.y
        if y > ground:
            y_prev = sk.y
            y = 2 * y_prev - self.wheel_y_prev + C.GRAVITY * (C.GRAVITY_TIMESTEP_2)
            self.wheel_y_prev = y_prev

            # bounce
            if y < ground and self.wheel_y_prev - y > C.BOUNCE_THRESHOLD:
                y = 2 * ground - y
                self.wheel_y_prev = ground - (self.wheel_y_prev -
                    ground) * C.BOUNCE_DAMPENING
            elif y < ground:
                y = self.wheel_y_prev = ground

        sk.set_position(x, y)

        # Wheel rotation
        wheel._rotation = (wheel.x + self.wheel_offset) / -wheel.circumference * math.pi

        # Seat rotation
        seat_force = self.wheel_dx / 50
        sk['Seat']._rotation = math.asin(max(-1, min(1, seat_force)))
        sk['Torso']._rotation = -2 * sk['Seat']._rotation

        # Pedal positions
        c = math.cos(wheel._rotation)
        s = math.sin(wheel._rotation)
        pedal_dist = wheel.radius * .45
        pedal_dx = c * pedal_dist - s * pedal_dist
        pedal_dy = s * pedal_dist + c * pedal_dist
        sk['LeftFoot'].ik(wheel._x + pedal_dx, wheel._y + pedal_dy)
        sk['RightFoot'].ik(wheel._x - pedal_dx, wheel._y - pedal_dy)

        self.left_hand_ik = False
        self.right_hand_ik = False

        # Continue object activation or activate something new
        if not self.active_object or not self.active_object.activate():
            for obj in glob.window.objects:
                if obj.activate():
                    self.active_object = obj
                    break
            else:
                self.active_object = None

        # Rest hands if not IKing to something else
        if not self.right_hand_ik:
            self.ik_action('RightHand', 'Rest', True)
        if not self.left_hand_ik:
            self.ik_action('LeftHand', 'Rest', True)

        self.skeleton.update(dt)
        if self.has_plate:
            self.plate._update_position()

    def draw(self):
        self.skeleton.draw()

    def facing(self, other):
        return self.skeleton.flip_x == (self.skeleton.x > other.x)

    def flip_sign(self):
        return -1 if self.skeleton.flip_x else 1

    def ik_right_hand(self, x, y, speed=1.):
        self.skeleton['RightHand'].ik(x, y, True, speed)
        self.right_hand_ik = True

    def ik_left_hand(self, x, y, speed=1.):
        self.skeleton['LeftHand'].ik(x, y, True, speed)
        self.left_hand_ik = True

    def ik_action(self, bone_name, action, flip_knee=False, speed=.2):
        sk = self.skeleton
        torso_x, torso_y = sk['Torso'].get_position()
        rel_x = getattr(man_vectors, 'IK' + bone_name + action + '_X')
        rel_y = getattr(man_vectors, 'IK' + bone_name + action + '_Y')
        sk[bone_name].ik(torso_x + self.flip_sign() * rel_x, 
                         torso_y + rel_y, 
                         True, speed)
        
