#!/usr/bin/env python

'''
'''

__docformat__ = 'restructuredtext'
__version__ = '$Id: $'

import pyglet

from gamelib.BoneSprite import BoneSprite
from gamelib import glob

import res
from res import audience_bones
from res import man_bones
from res import plate_bones
from res import hat_bones
from res import elephant_bones
from res import dog_bones
from res import seal_bones
from res import balloons_bones

class Skeleton(object):
    _flip_x = False

    def __init__(self, asset_name):
        self.asset_name = asset_name
        self.bone_data = getattr(res, '%s_bones' % asset_name)
        self.draw_order = []
        self.update_order = []
        self.bones = {}
        self.roots = []
        self.set_position(0, 0)

    def __getitem__(self, name):
        try:
            return self.bones[name]
        except KeyError:
            return self.add(name)
    
    def add(self, name):
        bone = getattr(self.bone_data, name)
        img = pyglet.resource.image('%s_%s.png' % (self.asset_name, name))
        img.anchor_x = bone[0]
        img.anchor_y = img.height - bone[1]
        sprite = BoneSprite(img, batch=glob.window.batch)
        sprite.init_bone(bone[2], -bone[3])
        sprite.name = name
        self.bones[name] = sprite
        self.draw_order.append(sprite)
        self.update_order.append(sprite)
        return sprite

    def root(self, bone_name):
        bone = self[bone_name]
        self.roots.append(bone)
        i = self.update_order.index(bone)
        del self.update_order[i]
        self.update_order.insert(0, bone)
    
    def connect(self, bone_name1, bone_name2):
        bone1 = self[bone_name1]
        bone2 = self[bone_name2]
        bone2.bone_parent = bone1
        i1 = self.update_order.index(bone1)
        i2 = self.update_order.index(bone2)
        if i2 < i1:
            del self.update_order[i1]
            self.update_order.insert(i2, bone1)

    def set_position(self, x, y):
        self.x = x
        self.y = y
        for root in self.roots:
            root._x = x
            root._y = y

    def draw(self): 
        for sprite in self.draw_order:
            sprite.draw()

    def update(self, dt):
        for sprite in self.update_order:
            sprite._update_position()

    def set_flip_x(self, flip):
        self._flip_x = flip
        for sprite in self.update_order:
            sprite.flip_x = flip
        
    flip_x = property(lambda self: self._flip_x, set_flip_x)


