#!/usr/bin/env python

'''
'''

__docformat__ = 'restructuredtext'
__version__ = '$Id: $'

import math

from gamelib.GameObject import GameObject
from gamelib.Skeleton import Skeleton

from gamelib import glob
from gamelib import constants as C

from res import man_vectors

class SpinningPlate(GameObject):
    did_purchase = None
    did_fall = None
    did_start = None
    
    def __init__(self, x):
        self.skeleton = sk = Skeleton('plate')
        self.base_x = x

        sk.root('Stick')
        sk.add('Plate')

        self.stick = sk['Stick']
        self.plate = sk['Plate']

        self.reset()

    def reset(self):
        self.plate.y = -50
        self.plate.bone_parent = None
        self.stable_timer = 0
        self.swing = 0
        self.deflection = C.PLATE_DEFLECTION_MAX
        self.active = False
        self.skeleton.set_position(-1000, C.GROUND)

        self.state = 'unconstructed'

    def construct(self):
        self.state = 'idle'
        self.skeleton.set_position(self.base_x, C.GROUND)
        if SpinningPlate.did_purchase:
            SpinningPlate.did_purchase()

    def update(self, dt):
        self.stable_timer += dt

        plate = self.plate
        stick = self.stick

        if self.active:
            self.deflection += C.PLATE_DEFLECTION_CHANGE_ACTIVE * dt
        elif self.stable_timer > C.PLATE_STABLE_TIME:
            self.deflection += C.PLATE_DEFLECTION_CHANGE * dt

        if self.deflection < 0:
            self.deflection = 0
            self.stable_timer = 0

        self.deflection = min(self.deflection, C.PLATE_DEFLECTION_MAX)

        plate_swing = C.PLATE_SWING_ACTIVE if self.active and self.deflection != 0 else C.PLATE_SWING
        self.swing += dt * math.pi * plate_swing * (1.5 - self.deflection / C.PLATE_DEFLECTION_MAX)

        stick.rotation = math.sin(self.swing) * self.deflection

        if self.state == 'falling' or self.state == 'landing':
            # verlet
            prev_x = plate._x
            plate.x = 2 * prev_x - self.plate_prev_x
            self.plate_prev_x = prev_x

            prev_y = plate._y
            plate.y = 2 * prev_y - self.plate_prev_y + \
                C.GRAVITY * C.GRAVITY_TIMESTEP_2
            self.plate_prev_y = prev_y
            plate.rotation += C.PLATE_FALL_DR * dt

            if plate.y < -50:
                self.state = 'idle'
                glob.window.play('glass_breaking_2.wav', over=True)
                if self.did_fall:
                    self.did_fall()
                if glob.lives == 0:
                    glob.window.is_game_over = True

        elif self.state == 'attached':
            plate.rotation = -stick.rotation + math.cos(2.5 * self.swing) / 10
            if self.deflection > C.PLATE_DEFLECTION_FALL:
                self.plate.x, self.plate.y = self.plate.get_position()
                self.plate_falling = True
                self.state = 'falling'
                plate.bone_parent = None
                self.plate_prev_x = self.plate.x - self.swing * C.PLATE_FALL_DX
                self.plate_prev_y = self.plate.y


        if self.state == 'attached':
            if self.active:
                glob.entertainment += C.ENTERTAINMENT_PLATE / 2
            else:
                glob.entertainment += C.ENTERTAINMENT_PLATE / 2
        elif self.state != 'unconstructed':
            glob.entertainment -= C.ENTERTAINMENT_PLATE

        self.skeleton.update(dt)


    def draw(self):
        self.skeleton.draw()
        
    def activate(self):
        player = glob.player
        flip = player.flip_sign()

        if (player.facing(self) and
            (player.has_plate or self.state != 'idle') and
            abs(player.wheel_dx) < C.PLATE_ACTIVATE_MAX_DX and
            flip * (player.x - self.x) > C.PLATE_ACTIVATE_MIN_X and
            flip * (player.x - self.x) < C.PLATE_ACTIVATE_MAX_X):

            grab_y = man_vectors.IKSpinningPlateGrab_Y + C.GROUND
            grab_x = self.x + grab_y * math.sin(-self.skeleton['Stick'].rotation) 
            player.ik_right_hand(grab_x, grab_y)

            if self.state == 'idle' and \
               self.deflection < C.PLATE_DEFLECTION_FALL:
                self.state = 'landing'
                self.plate.x = self.stick.x
                self.plate.y = player.skeleton['RightHand']._bone_next_y
                self.plate_prev_x = self.plate.x
                self.plate_prev_y = self.plate.y - C.PLATE_THROW_DY
                self.plate.rotation = C.PLATE_THROW_R
                glob.lives -= 1
                player.has_plate = False
                player.plate.bone_parent = None
                player.plate.y = -1000

            elif self.state == 'landing' and \
               self.plate.y < self.stick._bone_next_y and \
               self.plate_prev_y > self.plate.y:
                # reparent
                self.state = 'attached'
                glob.window.check_game_won()
                self.plate.bone_parent = self.stick
                self.plate.visible = True
                self.plate.rotation = 0
                if (all(p.state == 'attached' 
                       for p in glob.window.constructed_poles) and
                    len(glob.window.constructed_poles) > 2):
                    glob.window.play('small_exterior_crowd_applause_with_cheering.mp3', queue=False)
                if self.did_start:
                    self.did_start()

            elif self.state == 'landing':
                if self.plate.y < self.stick._bone_next_y:
                    player.ik_left_hand(self.plate.x, self.plate.y)

            self.active = True
            return True

        self.active = False
        return False
