#!/usr/bin/env python

'''
'''

__docformat__ = 'restructuredtext'
__version__ = '$Id: $'

import pyglet

from pyglet.window import key

from gamelib.Button import Button
from gamelib import constants as C
from gamelib import glob

class Store(object):
    def __init__(self):
        self.batch = pyglet.graphics.Batch()
        m = C.STORE_MARGIN
        self.bg = Button(m, m, glob.window.width - m, glob.window.height - m, self.batch) 
        self.bg.set_color(C.STORE_COLOR_BG)

        self.title = pyglet.text.Label('Purchase items from the store', 
                                       font_name=C.FONT_NAME,
                                       font_size=18., 
                                       x=glob.window.width/2,
                                       y=glob.window.height - m - C.STORE_PAD_Y,
                                       anchor_x='center',
                                       anchor_y='top',
                                       color=(0, 0, 0, 255),
                                       batch=self.batch)

        self.items = [
            Item(1, "Spinning Pole", C.PRICE_SPINNING_POLE, self.batch, 
                 glob.window.purchase_pole, 
                 lambda: glob.window.unconstructed_poles),
            Item(2, "Balloons", C.PRICE_BALLOONS, self.batch, 
                 glob.window.balloons.purchase,
                 lambda: not glob.window.balloons.purchased),
            Item(3, "Dog", C.PRICE_DOG, self.batch, 
                 glob.window.dog.purchase,
                 lambda: not glob.window.dog.purchased),
            Item(4, "Seal", C.PRICE_SEAL, self.batch, 
                 glob.window.seal.purchase,
                 lambda: not glob.window.seal.purchased),
            Item(5, "Elephant", C.PRICE_ELEPHANT, self.batch, 
                 glob.window.elephant.purchase,
                 lambda: not glob.window.elephant.purchased),
        ]

    def refresh(self):
        for item in self.items:
            item.refresh()
        self.select(len(self.items) - 1)
        self.move_select(1)

    def select(self, index):
        self.selected_index = index
        for i, item in enumerate(self.items):
            if i == index and item.state != 'disabled':
                item.set_state('selected')
            elif item.state == 'selected':
                item.set_state('idle')

    def move_select(self, dir):
        start = self.selected_index % len(self.items)
        while True:
            self.selected_index = (self.selected_index + dir) % len(self.items)
            if self.selected_index == start:
                break
            elif self.items[self.selected_index].state != 'disabled':
                self.select(self.selected_index)
                break

    def draw(self):
        self.batch.draw()

    def on_key_press(self, symbol, modifiers):
        if symbol == key.SPACE or symbol == key.ESCAPE:
            glob.window.pop_dialog()
        elif symbol == key.DOWN:
            self.move_select(1)
        elif symbol == key.UP:
            self.move_select(-1)
        elif symbol == key.ENTER:
            if self.selected_index != -1:
                self.items[self.selected_index].purchase()
                glob.window.pop_dialog()
        return True

    def on_mouse_motion(self, x, y, dx, dy):
        for i, item in enumerate(self.items):
            if item.hit_test(x, y):
                self.select(i)
                break
        else:
            self.select(-1)

    def on_mouse_press(self, x, y, button, modifiers):
        for i, item in enumerate(self.items):
            if item.hit_test(x, y) and item.state != 'disabled':
                item.purchase()
                glob.window.pop_dialog()
                break

class Item(object):
    def __init__(self, i, name, price, batch, func, condition=None):
        self.price = price
        self.func = func
        self.condition = condition
        self.name = name

        y2 = glob.window.height - C.STORE_MARGIN - C.STORE_PAD_Y - \
            i * (C.STORE_ITEM_LEADING + C.STORE_ITEM_HEIGHT)
        y1 = y2 - C.STORE_ITEM_HEIGHT
        x1 = C.STORE_MARGIN + C.STORE_PAD_X
        x2 = glob.window.width - C.STORE_MARGIN - C.STORE_PAD_X
        self.x1 = x1
        self.y1 = y1
        self.x2 = x2
        self.y2 = y2
        self.button = Button(x1, y1, x2, y2, batch)

        self.label = pyglet.text.Label('(Locked)',
                                       font_name=C.FONT_NAME,
                                       font_size=20., 
                                       anchor_x='left',
                                       anchor_y='center',
                                       color=(0, 0, 0, 255),
                                       batch=batch)
        self.label.x = x1 + C.STORE_ITEM_PAD_X
        self.label.y = (y1 + y2) / 2 + 4 # fudge vcenter

        self.price_label = pyglet.text.Label('$%0.2f' % price,
                                       font_name=C.FONT_NAME,
                                       font_size=16., 
                                       anchor_x='right',
                                       anchor_y='center',
                                       color=(0, 0, 0, 255),
                                       batch=batch)

        self.price_label.x = x2 - C.STORE_ITEM_PAD_X
        self.price_label.y = (y1 + y2) / 2 + 4 # fudge vcenter

        self.set_state('idle')

    def set_state(self, state):
        self.state = state
        if state == 'idle':
            self.button.set_color(C.STORE_ITEM_COLOR_BG)
        elif state == 'selected':
            self.button.set_color(C.STORE_ITEM_COLOR_BG_SELECTED)
        elif state == 'disabled':
            self.button.set_color(C.STORE_ITEM_COLOR_BG_DISABLED)
        
    def refresh(self):
        if (self.condition and not self.condition()) or self.price > glob.cash:
            self.set_state('disabled')
        else:
            self.set_state('idle')
            self.label.text = self.name

    def purchase(self):
        if self.state != 'disabled':
            glob.window.play('antique_cash_register_punching_single_key.mp3', over=True)
            glob.cash -= self.price
            self.func()
            glob.window.check_game_won()

    def hit_test(self, x, y):
        return (self.x1 < x < self.x2) and (self.y1 < y < self.y2)
