#!/usr/bin/env python

'''
'''

__docformat__ = 'restructuredtext'
__version__ = '$Id: $'

try:
    import pyglet.media.avbin
    have_avbin = True
except:
    have_avbin = False

import math
import os
import sys

import pyglet
pyglet.options['debug_gl'] = False

from pyglet.gl import *
from pyglet.window import key

from gamelib.Animal import Animal
from gamelib.Audience import Audience
from gamelib.BoneSprite import BoneSprite
from gamelib.Coin import Coin
from gamelib.Dialog import Dialog
from gamelib.Hat import Hat
from gamelib.Hints import Hints
from gamelib.Plate import Plate
from gamelib.Player import Player
from gamelib.SpinningPlate import SpinningPlate
from gamelib.Store import Store
from gamelib.Title import Title

from gamelib import constants as C
from gamelib import glob

import res
from res import background_vectors

pyglet.resource.path = ['@res']
pyglet.resource.reindex()

def sign(x):
    return 1 if x > 0 else -1

# monkey-patch some pixels of padding into texture atlas:
def _texture_atlas_add(self, img):
    pad = 1
    x, y = self.allocator.alloc(img.width + pad * 2, img.height + pad * 2)
    self.texture.blit_into(img, x + pad, y + pad, 0)
    region = self.texture.get_region(x + pad, y + pad, img.width, img.height)
    return region 
pyglet.image.atlas.TextureAtlas.add = _texture_atlas_add

class Window(pyglet.window.Window):
    def init(self):
        pyglet.resource.add_font('Baramond.ttf')

        self.bg_batch = pyglet.graphics.Batch()
        self.bg = BoneSprite(pyglet.resource.image('bg.png'),
                             batch=self.bg_batch)
        self.palm = BoneSprite(pyglet.resource.image('palm.png'),
                               batch=self.bg_batch)
        self.palm.x = background_vectors.palm_X
        self.palm.y = background_vectors.palm_Y

        self.dialogs = []

        self.batch = pyglet.graphics.Batch()

        # All these resources go into the same texture atlas
        bin = pyglet.image.atlas.TextureBin(512, 512)
        def _get_texture_atlas_bin(self, width, height):
            return bin
        restore_get_texture_atlas_bin = pyglet.resource.Loader._get_texture_atlas_bin
        pyglet.resource.Loader._get_texture_atlas_bin = _get_texture_atlas_bin

        # force a packing order (tall objects on first row)
        # todo fix pyglet's packer
        pack_order = [
            pyglet.resource.image('plate_Stick.png'),
            pyglet.resource.image('elephant_Root.png'),
            pyglet.resource.image('balloons_Red.png'),
            pyglet.resource.image('balloons_Yellow.png'),
            pyglet.resource.image('seal_Root.png'),
            pyglet.resource.image('dog_Root.png'),
        ]   

        self.objects = []

        self.ship = BoneSprite(pyglet.resource.image('ship.png'),
                               batch=self.batch)
        self.ship.x = -50
        self.ship.y = background_vectors.ship_Y

        self.elephant = Animal('elephant')
        self.objects.append(self.elephant)

        for i in range(20):
            self.objects.append(Audience(100 - i * 5))

        self.dog = Animal('dog')
        self.objects.append(self.dog)

        self.unconstructed_poles = [
            SpinningPlate(400),
            SpinningPlate(200),
            SpinningPlate(600),
            SpinningPlate(300),
            SpinningPlate(500),
            SpinningPlate(700),
        ]
        self.constructed_poles = []
        map(self.objects.append, self.unconstructed_poles)

        self.balloons = Animal('balloons')
        self.objects.append(self.balloons)

        self.hat = Hat()
        self.objects.append(self.hat)

        self.lives = []
        for i in range(C.LIVES_MAX):
            self.lives.append(Plate(i))
            self.objects.append(self.lives[-1])

        glob.player = Player()

        self.push_handlers(glob.player.keys)
        self.objects.append(glob.player)

        self.coins = []
        for i in range(C.COINS_MAX):
            self.coins.append(Coin())
            self.objects.append(self.coins[-1])

        self.seal = Animal('seal')
        self.objects.append(self.seal)

        # Resume regular texture atlas binning
        pyglet.resource.Loader._get_texture_atlas_bin = restore_get_texture_atlas_bin 
        #bin.atlases[0].texture.save('atlas.png')

        self.store = Store()

        pyglet.clock.schedule_interval(self.on_update, 1 / 30.)
        self.hints = Hints()

        self.goto_title()

        if have_avbin:
            self.music_player = pyglet.media.Player()
            self.music_player.queue(pyglet.resource.media('crowdy_street.mp3', streaming=True))
            self.music_player.play()
            self.music_player.eos_action = 'loop'

        self.player = pyglet.media.Player()

    def play(self, sfx, queue=True, over=False):
        if sfx.endswith('.mp3') and not have_avbin:
            return 
        if over:
            pyglet.resource.media(sfx).play()
        elif queue or not self.player.playing:
            self.player.queue(pyglet.resource.media(sfx))
            self.player.play()

    def on_draw(self):
        self.bg_batch.draw()

        if not self.dialogs or self.dialogs[0].__class__.__name__ != 'Title':
            self.batch.draw()
            self.hat.draw()

        self.hints.draw()

        for dialog in self.dialogs:
            dialog.draw()

    def on_update(self, dt):
        self.ship.x += dt * 2

        if self.dialogs:
            pass
        else:
            dt = min(dt, C.TIMESTEP_MAX)
            
            glob.last_entertainment = glob.entertainment
            glob.entertainment = C.ENTERTAINMENT_BASE
            map(lambda o: o.update(dt), self.objects)

            if self.is_game_over:
                self.game_over()

    def on_mouse_drag(self, x, y, dx, dy, buttons, modifiers):
        #self.man['LeftFoot'].ik(x, y)
        pass

    def push_dialog(self, dialog):
        self.dialogs.append(dialog)
        self.push_handlers(dialog)

    def pop_dialog(self):
        del self.dialogs[-1]
        self.pop_handlers()

    def on_key_press(self, symbol, modifiers):
        if symbol == key.SPACE:
            self.store.refresh()
            self.push_dialog(self.store)
            if self.did_store:
                self.did_store()
        elif symbol == key.ESCAPE:
            dialog = Dialog('Quit Game?')
            dialog.add('Yes, quit.', pyglet.app.exit)
            dialog.add('No, keep cycling!', lambda: None)
            self.push_dialog(dialog)
        elif C.CHEAT:
            if symbol == key.R:
                reload(C)
                print 'reload'
            elif symbol == key.C:
                glob.cash += 20
            elif symbol == key.G:
                self.game_over()

    def purchase_pole(self):
        if self.unconstructed_poles:
            if len(self.unconstructed_poles) == 6: # first pole
                glob.lives += 3
            else:
                glob.lives += 1
            pole = self.unconstructed_poles.pop(0)
            pole.construct()
            self.constructed_poles.append(pole)
        
    def game_over(self):
        dialog = Dialog('Game Over')
        dialog.add('Play again', self.goto_title)
        dialog.add('Quit', pyglet.app.exit)
        self.push_dialog(dialog)

    def goto_title(self):
        self.play_again()
        self.push_dialog(Title())

    def play_again(self):
        self.is_game_over = False
        glob.cash = C.CASH_BEGIN
        glob.lives = C.LIVES_BEGIN
        glob.entertainment = C.ENTERTAINMENT_BASE
        glob.audience_count = 0
        glob.audience_standing_count = 0 
        self.unconstructed_poles = self.constructed_poles + self.unconstructed_poles
        self.constructed_poles=  []
        self.hints.reset()

        glob.player.skeleton.set_position(300, glob.player.wheel.radius + C.GROUND)
        for object in self.objects:
            object.reset()

    def check_game_won(self):
        if (not self.unconstructed_poles and
            all(p.state == 'attached' for p in self.constructed_poles) and
            self.balloons.purchased and
            self.dog.purchased and
            self.seal.purchased and
            self.elephant.purchased):
            self.hints.set('Congratulations! You have won the game')


if __name__ == '__main__':
    glob.window = Window(800, 600, 'The Street Performer')
    glob.window.init()
    pyglet.app.run()
